// Insert copyright notice for plugin here.
// Note that plugins using this template or linking to libbase are
// derivative work as defined in the GNU General Public License, and
// as such they are also covered by the license.
//
// This template is based on the medianratio plugin. It might need
// lots of changes to fit your project.

//
// This is a plugin for BASE - http://base.thep.lu.se/
// Copyright (C) 2002,2003 Lao Saal, Carl Troein, Johan Vallon-Christersson,
//  Bjrn Samuelsson, YOUR_NAME_NERE
//
// This is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <cstdlib>
#include <cstring>
#include <cstdarg>
#include <iostream>
#include <map>
#include <vector>
#include <algorithm>

#include <basefile_plugin.h>
#include "microarr_point.hh"

using namespace std;
using namespace BASE;

// Construct a vector<string> from a list of char pointers
vector<string> makesvec(char *str, ...);

// Print any error messages from the reader, and then return false.
bool printErr(PluginBaseFileReader &pbfr);

// Process data in one way or another (may need renaming...)
bool normalize(PluginBaseFileReader& pbfr);

// This is used in some error messages
char *progname;



bool dothings()
{
	// Set up a PluginBaseFileReader from cin
	PluginBaseFileReader pbfr(cin);

	// A list of the settings this plugin requires
	vector<string> settings = makesvec("some_option", NULL);
	if(!pbfr.registerSettings("myplugin settings", PLUGIN_VERSION, settings))
		return printErr(pbfr);

	// This plugin requires columns "position" and "reporter" and
	// fields "intensity1" and "intensity2", and can only operate
	// on 2-channel data in the serial format.
	vector<string> columns = makesvec("position", "reporter", NULL);
	vector<string> fields = makesvec(NULL);

	// In the future it's possible that this function may return false.
	if(!pbfr.registerDataFormat(columns, fields, true, 2, 2,
		PluginBaseFileReader::format_serial))
	{
		return printErr(pbfr);
	}

	// Read plugin settings, assay annotations, and whatnot.
	if(!pbfr.readInfoSections())
		return printErr(pbfr);

	// Beginning of output for a plugin that transforms data
	// or outputs a reporter list
	cout << "BASEfile\n";

	// Loop over the file until no more sections can be read
	while(pbfr.readSpotsSection())
	{
		// Some output specific to a transforming plugins (on 2 channels).
		cout << "section\tspots\n"
			"columns\tposition\treporter\tassayData\n"
			"assayFields\tintensity1\tintensity2\n"
			"assays\t" << pbfr.getAssay(0).id << "\n"
			"%\n";

		if(!normalize(pbfr))
			return false;

		// Terminate the section we just started to output.
		cout << "\n";
	}
	return pbfr.getError() ? printErr(pbfr) : true;
}


// This function doesn't normalize anything. But at least it gets the
// data into a format that would be easy to write a normalizer for.
bool normalize(PluginBaseFileReader &pbfr)
{
	// This is how you get (and sanity check) a plugin setting
	double something = pbfr.getSettingDouble("some_option");
	if(something < 0. || something > 1.)
	{
		cerr << progname << ": Invalid something setting (" <<
			something << "). Please make sense." << endl;
	  return false;
	}

	// See microarr_point.hh
	vector<microarr_point> data;
	typedef vector<microarr_point*>::iterator microarr_ptr_iter;
	// Map from position to reporter (we're only dealing with
	// a single assay, so this is unambigous).
	map<int,int> posRep;

	// Read the entire section into the vector 'data'.
	while(pbfr.readDataLine())
	{
		int pos = pbfr.getColumnInt("position");
		data.push_back(microarr_point(pos,
			pbfr.getIntensity(0, 1), pbfr.getIntensity(0, 2)));
		posRep[pos] = pbfr.getColumnInt("reporter");
	}

	// Here you could do some sort of analysis...
	// int npoints = data.end() - data.begin();

	// Output the data unchanged - this is just an example, after all.
	for(vector<microarr_point>::iterator iter = data.begin();
		iter != data.end(); iter++)
	{
		cout << iter->index << "\t" <<
			posRep[iter->index] << "\t" <<
			iter->R << "\t" <<
			iter->G << "\n";
	}

	return true;
}



int main(int argc, char **argv)
{
#ifdef HAVE_PUBSETBUF
	// With some compiler/libstdc++ versions setting the output buffer size
	// is know to give a significant performance boost.
	char coutbuf[4096];
	cout.rdbuf()->pubsetbuf(coutbuf, sizeof(coutbuf));
#endif

	if(argc)
		progname = argv[0];
	else
		progname = "unknown";

	int ok = dothings();

#ifdef HAVE_PUBSETBUF
	// We mustn't forget to free the buffer before returning,
	// or bad things may happen.
	cout.rdbuf()->pubsetbuf(NULL, 0);
#endif

	return !ok;
}


// Construct a vector<string> from a list of char pointers
vector<string> makesvec(char *str, ...)
{
	va_list va;
	va_start(va, str);
	vector<string> vec;
	if(str)
	{
		vec.push_back(string(str));
		char *s;
		while((s = va_arg(va, char *)))
			vec.push_back(string(s));
	}
	va_end(va);
	return vec;
}

// Print error msgs
bool printErr(PluginBaseFileReader &pbfr)
{
	if(pbfr.getError())
	{
		cerr << progname << ": " << pbfr.errText() << endl;
		string s = pbfr.errTextMore();
		if(!s.empty())
			cerr << "  " << s << endl;
	}
	else
		cerr << progname << ": " << "unknown error" << endl;
	return false;
}


